package cn.com.ctop.alarm.modules.constant;

import cn.com.ctop.alarm.modules.entity.RuleGroup;
import cn.com.ctop.common.module.utils.Check;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * TODO
 *
 * @ClassName MatchLogic
 * @Author ZHAOXA
 * @date 2020-11-17 10:54
 */
@Slf4j
public class MatchLogic {

    /**
     * @param type      数据类型
     * @param condition 条件
     * @param threshold 阈值
     * @param value     比对值
     * @return boolean
     * @throws
     * @author ZHAOXA
     */
    public static boolean matchCondition(String type, String condition, String threshold, String value) {
        try {
            if ("number".equals(type)) {
                if (Check.isNull(threshold) || Check.isNull(value)) {
                    return false;
                }
                BigDecimal bigThr = new BigDecimal(threshold);
                BigDecimal bigVal = new BigDecimal(value);
                switch (condition) {
                    case "equal":
                        return bigVal.compareTo(bigThr) == 0;
                    case "not_equal":
                        return bigVal.compareTo(bigThr) != 0;
                    case "greater":
                        return bigVal.compareTo(bigThr) > 0;
                    case "less":
                        return bigVal.compareTo(bigThr) < 0;
                    case "greater_equal":
                        return bigVal.compareTo(bigThr) >= 0;
                    case "less_equal":
                        return bigVal.compareTo(bigThr) <= 0;
                    default:
                        log.warn("关系不匹配");
                        break;
                }
            } else if ("string".equals(type)) {
                switch (condition) {
                    case "equal":
                        return value.equals(threshold);
                    case "not_equal":
                        return !value.equals(threshold);
                    case "contain":
                        return !value.contains(threshold);
                    case "no_contain":
                        return !value.contains(threshold);
                    default:
                        log.warn("关系不匹配");
                        break;
                }
            } else {
                List<String> thresholdArr = strToList(threshold);
                List<String> values = strToList(value);
                if ("contain".equals(condition)) {
                    for (String t : thresholdArr) {
                        for (String v : values) {
                            if (v.contains(t)) {
                                return true;
                            }
                        }
                    }
                } else {
                    for (String t : thresholdArr) {
                        for (String v : values) {
                            if (!v.contains(t)) {
                                return true;
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("条件匹配异常", e);
        }
        return false;
    }

    /**
     * 整理发送的消息
     */
    public static String getMsg(RuleGroup ruleGroup, JSONObject user, Long accountId, Long planId, Long creativeId, boolean isPause) {
        StringBuffer header = new StringBuffer();
        header.append(ruleGroup.getGroupName()).append("预警").append("<br/>").
                append("预警账户：").append(accountId).append("<br/>");
        if (!Check.isNull(planId) && !Check.isNull(creativeId)) {
            header.append("预警计划：").append(planId).append("<br/>").append("预警创意：").append(creativeId).append("<br/>");
        } else if (!Check.isNull(planId) && Check.isNull(creativeId)) {
            header.append("预警计划：").append(planId).append("<br/>");
        } else if (!Check.isNull(creativeId)) {
            header.append("预警创意：").append(creativeId).append("<br/>");
        }
        if (isPause) {
            header.append("现已被关停，请您及时查看并调整！");
        } else {
            header.append("请您及时查看并调整！");
        }
        return header.toString();
    }

    //拆分逗号拼接数据
    public static List<String> strToList(String strings) {
        List<String> ids = null;
        try {
            if (Check.isNull(strings)) {
                return null;
            }
            JSONObject job = new JSONObject();
            job.put("list", strings);
            ids = new ArrayList<>();
            JSONArray idList = job.getJSONArray("list");
            for (Object id : idList) {
                ids.add(String.valueOf(id));
            }
        } catch (Exception e) {
            log.error("数据（{}）转list异常", strings, e);
        }
        return ids;
    }

    public static JSONArray getSendData(JSONArray accountDatas, JSONArray planDatas, JSONArray creativeDatas, JSONArray targetDatas) {
        JSONArray sendData = new JSONArray();
        if (!Check.isNull(accountDatas) && Check.isNull(planDatas) && Check.isNull(creativeDatas) && Check.isNull(targetDatas)) {
            sendData = accountDatas;
        } else {
            if (!Check.isNull(planDatas) && Check.isNull(creativeDatas) && Check.isNull(targetDatas)) {
                sendData = planDatas;
            } else if (!Check.isNull(creativeDatas) && Check.isNull(planDatas) && Check.isNull(targetDatas)) {
                sendData = creativeDatas;
            } else if (!Check.isNull(targetDatas) && Check.isNull(planDatas) && Check.isNull(creativeDatas)) {
                sendData = targetDatas;
            } else if (!Check.isNull(planDatas) && !Check.isNull(creativeDatas) && !Check.isNull(targetDatas)) {
                for (int i = 0; i < planDatas.size(); i++) {
                    Long pplanId = planDatas.getJSONObject(i).getLong("planId");
                    for (int k = 0; k < targetDatas.size(); k++) {
                        Long tplanId = targetDatas.getJSONObject(k).getLong("planId");
                        for (int j = 0; j < creativeDatas.size(); j++) {
                            Long cplanId = creativeDatas.getJSONObject(j).getLong("planId");
                            if (cplanId == tplanId && cplanId == pplanId) {
                                sendData.add(creativeDatas.getJSONObject(j));
                            }
                        }
                    }
                }
            } else if (!Check.isNull(planDatas) && !Check.isNull(targetDatas) && Check.isNull(creativeDatas)) {
                for (int i = 0; i < planDatas.size(); i++) {
                    Long pplanId = planDatas.getJSONObject(i).getLong("planId");
                    for (int k = 0; k < targetDatas.size(); k++) {
                        Long tplanId = targetDatas.getJSONObject(k).getLong("planId");
                        if (tplanId == pplanId) {
                            sendData.add(targetDatas.getJSONObject(k));
                        }
                    }
                }
            } else if (!Check.isNull(planDatas) && Check.isNull(targetDatas) && !Check.isNull(creativeDatas)) {
                for (int i = 0; i < planDatas.size(); i++) {
                    Long pplanId = planDatas.getJSONObject(i).getLong("planId");
                    for (int k = 0; k < creativeDatas.size(); k++) {
                        Long cplanId = creativeDatas.getJSONObject(k).getLong("planId");
                        if (cplanId == pplanId) {
                            sendData.add(creativeDatas.getJSONObject(k));
                        }
                    }
                }
            } else if (Check.isNull(planDatas) && !Check.isNull(targetDatas) && !Check.isNull(creativeDatas)) {
                for (int i = 0; i < targetDatas.size(); i++) {
                    Long tplanId = targetDatas.getJSONObject(i).getLong("planId");
                    for (int k = 0; k < creativeDatas.size(); k++) {
                        Long cplanId = creativeDatas.getJSONObject(k).getLong("planId");
                        if (cplanId == tplanId) {
                            sendData.add(creativeDatas.getJSONObject(k));
                        }
                    }
                }
            }
        }
        return sendData;


    }


}
